# -*- coding: utf-8 -*-

from rest_framework import serializers

from thx.api.serializers import (
    ThuxListSerializerMixin, ThuxRetrieveSerializerMixin, ThuxCreateSerializerMixin,
    ThuxUpdateSerializerMixin, ThuxPartialUpdateSerializerMixin, ThuxStatusSerializerMixin,
    ThuxEnableSerializerMixin, ThuxDisableSerializerMixin
)
from thx.role.core.serializers import ThuxRoleSerializerMixin

from ... import models


# NOTIFICATION
class NotificationSerializer(ThuxRoleSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification'
        model = models.Notification
        fields = '__all__'


class NotificationListSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_list'
        model = models.Notification
        fields = '__all__'


class NotificationRetrieveSerializer(ThuxRoleSerializerMixin, ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_retrieve'
        model = models.Notification
        fields = '__all__'


class NotificationCreateSerializer(ThuxRoleSerializerMixin, ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_create'
        model = models.Notification
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def to_representation(self, instance):
        return NotificationListSerializer(instance, context=self.context).data


class NotificationUpdateSerializer(ThuxRoleSerializerMixin, ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_update'
        model = models.Notification
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def to_representation(self, instance):
        return NotificationRetrieveSerializer(instance, context=self.context).data


class NotificationPartialUpdateSerializer(
    ThuxRoleSerializerMixin, ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_notify_notification_partial_update'
        model = models.Notification
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def to_representation(self, instance):
        return NotificationRetrieveSerializer(instance, context=self.context).data


class NotificationEnableSerializer(ThuxRoleSerializerMixin, ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_enable'
        model = models.Notification
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance

    def to_representation(self, instance):
        return NotificationRetrieveSerializer(instance, context=self.context).data


class NotificationDisableSerializer(ThuxRoleSerializerMixin, ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_disable'
        model = models.Notification
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance

    def to_representation(self, instance):
        return NotificationRetrieveSerializer(instance, context=self.context).data


# NOTIFICATION RECIPIENT
class NotificationRecipientSerializer(ThuxRoleSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_recipient'
        model = models.NotificationRecipient
        fields = '__all__'


class NotificationRecipientListSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_list'
        model = models.NotificationRecipient
        fields = '__all__'


class NotificationRecipientRetrieveSerializer(
    ThuxRoleSerializerMixin, ThuxRetrieveSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_retrieve'
        model = models.NotificationRecipient
        fields = '__all__'


class NotificationRecipientCreateSerializer(
    ThuxRoleSerializerMixin, ThuxCreateSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_create'
        model = models.NotificationRecipient
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def to_representation(self, instance):
        return NotificationRecipientListSerializer(instance, context=self.context).data


class NotificationRecipientUpdateSerializer(
    ThuxRoleSerializerMixin, ThuxUpdateSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_update'
        model = models.NotificationRecipient
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def to_representation(self, instance):
        return NotificationRecipientRetrieveSerializer(instance, context=self.context).data


class NotificationRecipientPartialUpdateSerializer(
    ThuxRoleSerializerMixin, ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_partial_update'
        model = models.NotificationRecipient
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def to_representation(self, instance):
        return NotificationRecipientRetrieveSerializer(instance, context=self.context).data


class NotificationRecipientEnableSerializer(
    ThuxRoleSerializerMixin, ThuxEnableSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_enable'
        model = models.NotificationRecipient
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance

    def to_representation(self, instance):
        return NotificationRecipientRetrieveSerializer(instance, context=self.context).data


class NotificationRecipientDisableSerializer(
    ThuxRoleSerializerMixin, ThuxDisableSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_disable'
        model = models.NotificationRecipient
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance

    def to_representation(self, instance):
        return NotificationRecipientRetrieveSerializer(instance, context=self.context).data


# NOTIFICATION TYPE
class NotificationTypeSerializer(ThuxRoleSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_type'
        model = models.NotificationType
        fields = '__all__'


class NotificationTypeListSerializer(ThuxRoleSerializerMixin, ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_type_list'
        model = models.NotificationType
        fields = '__all__'


class NotificationTypeRetrieveSerializer(
    ThuxRoleSerializerMixin, ThuxRetrieveSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_notify_notification_type_retrieve'
        model = models.NotificationType
        fields = '__all__'


class NotificationTypeCreateSerializer(
    ThuxRoleSerializerMixin, ThuxCreateSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_notify_notification_type_create'
        model = models.NotificationType
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def to_representation(self, instance):
        return NotificationTypeListSerializer(instance, context=self.context).data


class NotificationTypeUpdateSerializer(
    ThuxRoleSerializerMixin, ThuxUpdateSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_notify_notification_type_update'
        model = models.NotificationType
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def to_representation(self, instance):
        return NotificationTypeRetrieveSerializer(instance, context=self.context).data


class NotificationTypePartialUpdateSerializer(
    ThuxRoleSerializerMixin, ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_notify_notification_type_partial_update'
        model = models.NotificationType
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def to_representation(self, instance):
        return NotificationTypeRetrieveSerializer(instance, context=self.context).data


class NotificationTypeEnableSerializer(
    ThuxRoleSerializerMixin, ThuxEnableSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_notify_notification_type_enable'
        model = models.NotificationType
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance

    def to_representation(self, instance):
        return NotificationTypeRetrieveSerializer(instance, context=self.context).data


class NotificationTypeDisableSerializer(
    ThuxRoleSerializerMixin, ThuxDisableSerializerMixin, serializers.ModelSerializer
):
    class Meta:
        ref_name = 'v1_notify_notification_type_disable'
        model = models.NotificationType
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance

    def to_representation(self, instance):
        return NotificationTypeRetrieveSerializer(instance, context=self.context).data

